
// Copyright (c) 2010-2019 niXman (i dot nixman dog gmail dot com). All
// rights reserved.
//
// This file is part of YAS(https://github.com/niXman/yas) project.
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//
//
//
// Boost Software License - Version 1.0 - August 17th, 2003
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
//
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

#ifndef __yas__detail__io__exception_base_hpp
#define __yas__detail__io__exception_base_hpp

#include <yas/detail/preprocessor/stringize.hpp>

#include <stdexcept>

namespace yas {

/***************************************************************************/

#define __YAS_DECLARE_EXCEPTION_TYPE(type) \
    struct type: std::exception { \
        type(const char *msg) noexcept \
            :msg(msg) \
        {} \
        virtual ~type() noexcept {} \
        \
        virtual const char* what() const noexcept { return msg; } \
        \
    private: \
        const char *msg; \
    };

#define __YAS_EXCEPTION_MAKE_MSG(msg) \
    __FILE__ "(" YAS_PP_STRINGIZE(__LINE__) "): " msg

#define __YAS_THROW_EXCEPTION(type, msg) \
    throw type(__YAS_EXCEPTION_MAKE_MSG(msg))

/***************************************************************************/

} // ns yas

#endif // __yas__detail__io__exception_base_hpp
